#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#define N1 1000	/* Number of bootstrap samples */
#define N2 544  /* Sample size */
#define N3 60   /* Largest observation = infinity */
#define N4 4	/* Number of levels of the covariate */
#define TRUEBETA -1.0
#define TIME 20	/* 20 quarters or 5 years survival probability is to be esitmated */

/* Function Prototypes */

void MAKEDATA(register double[N2][3]);
void HHAT(register double[N2][3], register double[N4][N3+1][2]);
double HHH(register int, register double, register double[N2][3], register double[N4][N3+1][2]);
void INTEGRAL(register double[N2][N2], register double[N2][3], 
			  register double[N4][N3+1][2], register double[N2]);
double GAUSSNEWTON(double, register double[N2][N2],register double[N2][3]);
void F1_F2(double [2],double,register double [N2][N2],register double [N2][3]);

void COMPUTES(double, double, register double[N2], register double[N2][3], 
				register double[N4][N3+1][2], double[N1+1][N4+1], int);

void COMPUTEF(double[2], double[N2][3], double, double, double);

double MIN(double,double); 
double MAX(double,double);

void BOOTS(register double [N2][3], register double [N2][3]);
void SORTDATAINC(register double[N2][3], int, int);
int PARTITIONINC(register double[N2][3], int, int);

FILE *result;

void main(void)
{
	result=fopen("output.txt","w"); /* Result is printed to "output.txt" */
	
	srand((unsigned)time(NULL));
	
	register int i,j;
//	double time;					/* To be used in Summary 3 */
	double beta, est[N1+1][N4+1];	/* Results are stored in the matrix "est". The first column is beta */
				  					/* The next N4 columns are survival prob. for Z = 1,...,N4 */
									/* Row 1 is the original estimator, the rest are bootstrap results */
	register double data[N2][3];			/* Original data are stored in "data" */
	register double newdata[N2][3];			/* Bootstrap data are stored in "newdata" */
	register double hhat[N4][N3+1][2];		/* Turnbull estimate (Z=1,...,N4) are stored in "hhat" */

	register double ahat[N2];       /* Right-Left of hhat for observation 1,...,N2, depending on Z */
	register double hij[N2][N2];    /* Numerical integration result */

	double sum1=0.0,sum2=0.0;
	
	fprintf(result,"   Beta    ||  Survival Estimates (totally %d)\n", N4);
	fprintf(result," ==========|| ======================================= \n");

	MAKEDATA(data);					/* Get the original data */

	SORTDATAINC(data, 0, N2-1);		/* Sort data by Z */

	HHAT(data,hhat);				/* "hhat" store the Turnbull estimates */

	for (j=0;j<N2;j++)				/* "ahat" stores the Right-Left Turnbull estimates */
	{
		ahat[j]=HHH(j,data[j][2],data,hhat)-HHH(j,data[j][1],data,hhat);
	}

	INTEGRAL(hij,data,hhat,ahat);		/* Numerical integration, see paper for detail */ 
	beta=GAUSSNEWTON(TRUEBETA,hij,data);      /* Gauss-Newton algorithm is used to find the minimizer, TRUEBETA is the initial value */

	fprintf(result,"%f  ||  ",beta);

	est[0][0]=beta; 

	COMPUTES(TIME, beta, ahat, data, hhat, est, 0);

	for (j=1;j<=N4;j++)
	{
		fprintf(result,"%f  ", est[0][j]);
	}
	fprintf(result,"\n");
		
	for (i=1;i<=N1;i++)
	{
		printf("%d\n",i);
		
		BOOTS(newdata, data);			/* newdata is the bootstrap sample */								
		SORTDATAINC(newdata, 0, N2-1);	/* Sort newdata by Z */

		HHAT(newdata, hhat);			/* "hhat" store the Turnbull estimates */
		for (j=0;j<N2;j++)				/* "ahat" stores the Right-Left Turnbull estimates */
		{
			ahat[j]=HHH(j,newdata[j][2],newdata,hhat)-HHH(j,newdata[j][1],newdata,hhat);
		}
		INTEGRAL(hij,newdata,hhat,ahat);      /* Numerical integration, see paper for detail */ 
		beta=GAUSSNEWTON(TRUEBETA,hij,newdata);      /* Gauss-Newton algorithm is used to find the minimizer, TRUEBETA is the initial value */
		fprintf(result,"%f  ||  ", beta);
		est[i][0]=beta; 
	
		COMPUTES(TIME, beta, ahat, newdata, hhat, est, i);
	
		for (j=1;j<=N4;j++)
		{
			fprintf(result,"%f  ", est[i][j]);
		}
		fprintf(result,"\n");
	}
	fprintf(result,"\n\n");
	fprintf(result,"         Summary 1      \n");
	fprintf(result,"=============================================\n");
	fprintf(result,"Cox Proportional Hazards Model\n");
	fprintf(result,"Sample size is %d;   %d bootstrap replications\n\n\n",N2, N1);

	fprintf(result,"         Summary 2      \n");
	fprintf(result,"==================================================\n");
	fprintf(result,"Estimats are: %2.5f  %2.5f  %2.5f  %2.5f  %2.5f\n", est[0][0], est[0][1], est[0][2], est[0][3], est[0][4]);

	fprintf(result,"                      (Bootstrap) Summary 3      \n");
	fprintf(result,"=========================================================================\n");
	fprintf(result,"  Beta		S1		S2		S3		S4 \n\n\n");

	sum1=sum2=0;
	for (j=0;j<=N4;j++)
	{
		sum1=sum2=0;
		for (i=1;i<=N1;i++)
		{
			sum1 = sum1 + est[i][j];
			sum2 = sum2 + est[i][j] * est[i][j];
		}
		fprintf(result, "%f	", sum1/N1);			/* Print means */
	}
	fprintf(result, "\n");
	for (j=0;j<=N4;j++)
	{
		sum1=sum2=0;
		for (i=1;i<=N1;i++)
		{
			sum1 = sum1 + est[i][j];
			sum2 = sum2 + est[i][j] * est[i][j];
		}
		fprintf(result, "%f	", sqrt((sum2-sum1*sum1/N1)/(N1-1)) );	/* Print standard deviations */
	}

	fclose(result);
}

void COMPUTES(double t, double beta, register double ahat[N2], 
				register double data[N2][3], register double hhat[N4][N3+1][2],
				double est[N1+1][N4+1], int n)
{
/************************************************************
This function calculates the estimator of survival function
************************************************************/
	int i, count = 0;
	double A=0, Ff[2], diff = 1, h = 0, hnew;
	for (i=0;i<N2;i++)
	{
		if ( t > data[i][2] )		/* A is the empirical cdf */
		{
			A = A + 1.0;
		}
		else if ( t > data[i][1] )
		{
			if (ahat[i]>0)
			{
				A = A + (HHH(i,t,data,hhat)-HHH(i,data[i][1],data,hhat))/ahat[i];
			}
		}
	}
	if (A >= N2 )	h = 500;			/* positive infinity */
	else if (A <= 0 )	h = -500;		/* negative infinity */
	else
	{
		while (diff > 0.000000000001 && count < 20)
		{
			count = count + 1;
			COMPUTEF(Ff, data, beta, h, A);
			hnew = h - Ff[0]/Ff[1];
			diff = (hnew-h)*(hnew-h);
			h = hnew;
//			printf("%f\n",h);
		}
	}
	for (i=1;i<=N4;i++)
	{
		est[n][i] = exp ( -exp (h + beta*(i-1)) );
	}
}
	
void COMPUTEF(double Ff[2], double data[N2][3], double beta, double h, double A)
/***************************************************************
This function calculates the first derivative of F, which is f
***************************************************************/
{
	int i;
	double F= 0 - A, f=0;
	for (i=0;i<N2;i++)
	{
		F = F + 1 - exp(-exp(h+data[i][0]*beta));
		f = f + exp(-exp(h+data[i][0]*beta))*exp(h+data[i][0]*beta);
	}
	Ff[0]=F;
	Ff[1]=f;
}	
	
double GAUSSNEWTON(double initial,register double hij[N2][N2],register double data[N2][3])
{
/**********************************************************
This function uses Gauss-Newton algorithm and returns the
minimizer of the sum of squares.
**********************************************************/
	double value[2];                   /* First and second derivatives are stored in "value" */
	double diff=1,beta0=initial,beta1;
	int count=0;
	while (diff>0.0000000001 && count<10)
	{
		count++;
		F1_F2(value,beta0,hij,data);   /* First and second derivatives are evaluated at beta0 */
		beta1=beta0-value[0]/value[1]; /* Update the estimate */
		diff=pow((beta1-beta0),2);     /* Compute the difference */
		beta0=beta1;
	}
	return(beta0);
}  


void F1_F2(double value[2],double beta,register double hij[N2][N2],register double data[N2][3])
{
/**********************************************************
This function does not return anything but assigns values 
of the first and second derivatives evaluated at "beta" to 
the existed array "value". The original function is:
1/(1+exp((Zj-Zi)*beta)).
**********************************************************/
	int i,j;
	register double x,y,z,w,sum1=0.0,sum2=0.0;
	for(i=0;i<N2;i++)
	{
		for(j=0;j<N2;j++)
		{
			x=1.0 + exp(beta*(data[j][0]-data[i][0]));
			y=(data[j][0]-data[i][0])*(x-1.0);			/* thus if i=j then y is zero */
			z=hij[i][j]-1.0/x;                    
			w=y*(data[j][0]-data[i][0])*x;
			sum1=sum1+2.0*z*y/(x*x);
			sum2=sum2+2.0*(y*y/pow(x,4)+z*(w-2*y*y)/pow(x,3));					 
		}
	}
	value[0]=sum1; value[1]=sum2; /* sum1 is the first derivative and sum 2 is the second */
}

void INTEGRAL(register double hij[N2][N2],register double data[N2][3],
			  register double hhat[N4][N3+1][2],register double ahat[N2])
{
/**********************************************************
This function does not return anything but assigns the 
numerically integrated value to the existed matrix "hij".
See paper for detail.
**********************************************************/
	register int i, j, l;
	register int num; 
	register double sum;
	for (i=0;i<N2;i++)
	{
		for (j=0;j<N2;j++)
		{
			sum=0.0;
			l=1;

			num = static_cast<int>(data[i][0]) - 1;			/* Value of Z */
			
			while (l<=N3 && data[i][1]>=hhat[num][l][0])	l++;
			
			while (l<=N3 && data[i][2]>=hhat[num][l][0])
			{
				sum=sum+( HHH(j,MIN(data[j][2],hhat[num][l][0]),data,hhat)
						 -HHH(j,MIN(data[j][1],hhat[num][l][0]),data,hhat) )
					   *( HHH(i,hhat[num][l][0],data,hhat)
					     -HHH(i,hhat[num][l-1][0],data,hhat) );
				l++;                 /* See paper for detail */
			}
			
			if (i!=j && ahat[i]>0 && ahat[j]>0) 
			{
				hij[i][j]=sum/(ahat[i]*ahat[j]);
				if (hij[i][j]>1.0) hij[i][j]=1.0;
				if (hij[i][j]<0.0) hij[i][j]=0.0;
			}
			else	hij[i][j]=0.0;       /* Elements in "hij" are no larger than one */
										 /* actually when i=j, the estimating equation does not make sense */
		}
	}
}

double HHH(register int n, register double t, register double data[N2][3],
		   register double hhat[N4][N3+1][2])
{
/************************************************************
This function returns the cdf value at time t corresponding 
to the covariate of obsevation n.
************************************************************/

//	register int j=1;
	register int num = static_cast<int>(data[n][0]) ;	/* Value of Z = 1,...,N4 */
	register double value=0.0;

//	while (j<=hhat[num][0][0] && t>hhat[num][j][0])	
//	{
//		j++;
//	}
//	if (j>hhat[num][0][0])
//	{
//		value = 1.00;               /* t is bigger than the censoring */
//	}
//	else if (t<hhat0[j][0]) 
//	{
//		value=hhat[num][j-1][1];	/* t is between hhat[num][j-1][0] and hhat[num][j][0] */
//	}
//	else value=hhat[num][j][1];	    /* t is equal to hhat[num][j][0] */

	value = hhat[num-1][static_cast<int>(t)][1];

	return(value);
}

void HHAT(register double data[N2][3], register double hhat[N4][N3+1][2])
{
/***************************************************************
This function does not return anything but computes the 
Turnbull estimates (CDF) for different groups (Z=1,...,N4) and 
stores them in the existing tensor "hhat".
****************************************************************/
	
	register double time[N3+1];			/* Stores all timepoints */
//	register double temptime[N3+1];
	register int i, j, k, count[2][N4+1], tempz, tempt, start, end, indi;
	double x, diff=1.0, sum;

	for (i=0;i<=N3;i++)                     /* Initialization */
	{
		time[i]=-1;							/* timepoints are just 0 -- N3. But for */
	}										/* initialization they are set to be -1 first */
											/* Later we will count how many are not -1 */
	for (i=0;i<=N4;i++)                     /* Initialization */
	{
		count[0][i]=count[1][i]=0;			/* count[0][] will be used to count # of Z (or tempz). The first one is not used */
	}										/* count[1][] will be used to count # of timepoints. The first one is not used */
	
/* Part 1: Summarize Z */

	for (i=0;i<N2;i++)                     /* Start to count */
	{
		for (j=1;j<=N4;j++)                /* Z=1,...,N4 */
		{
			if ( fabs(data[i][0]-j) < 0.1 )		/* which means data[i][0] = j */
			{
				count[0][j] = count[0][j] + 1;	/* First row: subsample size for Z */
			}
		}
	}

	for (j=1;j<=N4;j++)			      /* Z=1,...,N4 */
	{

/* Part 2: Compute the indicator matrix "aaa", whose dimension is tempz*tempt. */

	/* 2.1. First find out for this Z, where to start and end in the data (already sorted by Z). */
		
		tempz = count[0][j];			/* Subset sample size */

		start = 0;						/* When Z = 1; initialization */
		end = count[0][1] - 1;			/* When Z = 1; initialization */

		for (i=2;i<=j;i++)
		{
			start = start + count[0][i-1];
			end = end + count[0][i];	
		}								/* Now we know where to start and end for this Z */

	/* 2.2 Find out how many distinct time points for this Z */

		for (i=0;i<=N3;i++)                     /* Re-initialization */
		{
			time[i]=-1;							
		}										

		for (i=start;i<=end;i++)		/* Fill the vector time with observed time points. */
		{
			time[ static_cast<int>(data[i][1]) ] = data[i][1];
			time[ static_cast<int>(data[i][2]) ] = data[i][2];
		}

		tempt = 0;						/* tempt will be the number of columns of aaa */

		for (i=0;i<=N3;i++)				/* Now count how many distinct time points are there for this Z */
		{
			if ( time[i] > -0.5 )		/* If > -1 then it has an observation since the initial value is -1. */
			{
				tempt = tempt + 1;
			}
		}								/* Not done yet! We still don't know if 0 and infty are included */

		if (time[0] < -0.5)				/* Means there is no zero, then we should add it in. */
		{
			tempt = tempt + 1;
		}

		if (time[N3] < -0.5)			/* Means there is no infty, then we should add it in. */
		{
			tempt = tempt + 1;			/* Totally tempt timepoints, including 0 and infty */
		}

//		for (i=0;i<=N3;i++)				/* Before modifying time, keep a copy as temptime */
//		{
//			temptime[i] = time[i];		/* temptime will be used in the end */
//		}

		time[0] = 0;					/* Start modifying time */

		indi = 1;						/* and we first set indi to be the first timepoint (other than 0) */
		for (i=1;i<=N3;i++)
		{
			if (time[i] > -0.5)			/* If this timepoint exists */
			{
				time[indi] = time[i];	/* then keep it at time[indi] */
				indi++;					/* and then increase indi by 1 */
			}
		}

		/* After this modification we only need to consider from time[0] to time[tempt-1] */

	/* 2.3 Define aaa to be a tempz * tempt matrix, and fill it with 0 and 1. */

		int **aaa=(int **)malloc(sizeof(int *)*tempz);

		for (i=0;i<tempz;i++)
		{
			aaa[i]=(int *)malloc(sizeof(int)*tempt);  /* tempz * tempt */
		}
	
		for (i=start;i<=end;i++)			/* For this Z, calculate aaa */
		{
//			fprintf(result,"\n");
			for (k=0;k<tempt;k++)
			{
				if (data[i][1]<time[k] && data[i][2]>=time[k]) 
				{
					aaa[i-start][k]=1;
				}
				else 
				{
					aaa[i-start][k]=0;	
				}
//				fprintf(result,"%d ",aaa[i-start][k]);
			}
		}
//			fprintf(result,"\n\n\n");


/* Part 3: Turnbull algorithm, for computing survival functions */

		double *temp=(double *)malloc(sizeof(double)*tempz);
		double *cdf=(double *)malloc(sizeof(double)*tempt);

		for (i=0;i<tempt;i++)
		{
			cdf[i] = 1.0 / tempt;	/* Initialization: every timepoint is given a mass of 1/tempt. */
		}
		
		diff = 1;

		while(diff>0.0000001)                       
		{
			diff=0.0;				/* diff will be calculated later */

			for (i=0;i<tempz;i++)
			{
				sum=0.0;
				for (k=0;k<tempt;k++)
				{
					sum=sum+aaa[i][k]*cdf[k]; 
				}
				temp[i]=sum;		/* This is the denominator alpha*p in the Turnbull algorithm for each i */
			}

			for (k=0;k<tempt;k++)
			{
				sum=0.0;
				for (i=0;i<tempz;i++)
				{
					sum=sum+aaa[i][k]/temp[i];
				}
				x=sum*cdf[k]/tempz;             /* x is the new (updated) estimator */
				diff=diff+fabs(x-cdf[k]);
				cdf[k]=x;
			}
		}
	
		for (i=1;i<tempt;i++)
		{
			cdf[i]=cdf[i]+cdf[i-1];				/* Compute the cumulative (cdf) */
			if (cdf[i]>1.0) cdf[i]=1.0;
			if (cdf[i]<0.0) cdf[i]=0.0;
//			fprintf(result, "%d  %f  %f\n", i, time[i], cdf[i]);
		}
		
		/* Now cdf has the prob. and time has the timepoints. We can update hhat. */

		indi = 1;		/* re-use indi, which indicates the position of hhat */
		
		hhat[j-1][0][0] = hhat[j-1][0][1] = 0;	/* Recall that Z = j now */
		
		for (i=1;i<tempt;i++)
		{
			if (time[i] > indi)				/* So there are "gaps" to fill */
			{
				for (k=indi; k<(static_cast<int>(time[i])); k++)
				{
					hhat[j-1][k][0] = k;
					hhat[j-1][k][1] = hhat[j-1][k-1][1];	/* cdf is right continuous so take the previous one */
				}
				hhat[j-1][static_cast<int>(time[i])][0] = time[i];
				hhat[j-1][static_cast<int>(time[i])][1] = cdf[i];
			}
			else
			{
				hhat[j-1][indi][0] = time[i];
				hhat[j-1][indi][1] = cdf[i];
			}
			indi = static_cast<int>(time[i]) + 1;	/* Re-set indi to be the next timepoint */
		}
	}	
}

double MIN(double num1,double num2)
{
/********************************************************
This function returns the smaller value of num1 and num2. 
********************************************************/
	if (num1<=num2)
		return(num1);
	else
		return(num2);
}

double MAX(double num1,double num2)
{
/********************************************************
This function returns the larger value of num1 and num2. 
********************************************************/
	if (num1>=num2)
		return(num1);
	else
		return(num2);
}

void MAKEDATA(register double matrix[N2][3])
{
/****************************************************************************************
This function does not return anything but modifies the existed matrix (array) called 
"data". The matrix "data" contains three columns and N2 rows (which means sample size
is N2). The first column is the covariate, Z and the next two columns contain the left 
and the right bounds of the observation time respectively. The following data were 
obtained from Bacchetti and Quale (2002, Biometrics, page 443 - 447). See 
"\ZZGMU\Phd\Paper1" and "ZZGMU\Phd\Miscelleneous\Datasets" for details. Here I took the 
simplied version -- Modified Dataset (of fixed.txt).
Z = 0: None, 1: Low, 2: Medium, 3: High; Will be changed later to 
Z = 1: None, 2: Low, 3: Medium, 4: High.
****************************************************************************************/

	int i;

	double left[N2]={
7, 9, 9, 0, 57, 23, 8, 1, 55, 55, 56, 54, 2, 53, 2, 55, 1, 20, 25, 0, 0, 24, 12, 
57, 26, 54, 7, 31, 56, 18, 56, 9, 9, 14, 54, 8, 56, 11, 55, 8, 54, 16, 7, 29, 7, 
55, 16, 56, 14, 57, 7, 2, 56, 55, 57, 15, 0, 18, 21, 11, 11, 9, 54, 56, 11, 14, 
54, 9, 19, 56, 5, 23, 17, 17, 54, 0, 54, 55, 57, 8, 31, 20, 11, 55, 20, 56, 6, 
56, 56, 52, 46, 45, 23, 13, 19, 49, 11, 56, 55, 53, 15, 52, 10, 6, 0, 0, 46, 16, 
57, 54, 57, 0, 57, 55, 54, 24, 20, 15, 1, 16, 53, 16, 43, 48, 40, 54, 2, 51, 56, 
11, 12, 56, 51, 18, 0, 0, 2, 2, 0, 55, 25, 54, 35, 35, 56, 42, 11, 19, 54, 10, 2, 
15, 53, 41, 50, 0, 23, 19, 45, 56, 0, 0, 0, 56, 56, 50, 54, 39, 56, 51, 0, 0, 0, 
55, 50, 45, 49, 52, 52, 57, 56, 55, 14, 54, 55, 55, 55, 0, 54, 56, 0, 55, 19, 55, 
53, 14, 6, 11, 0, 15, 2, 10, 13, 11, 54, 20, 17, 13, 29, 14, 15, 11, 0, 11, 54, 
16, 53, 16, 48, 55, 29, 14, 11, 16, 25, 46, 55, 54, 53, 53, 31, 40, 17, 10, 15, 
53, 20, 16, 20, 22, 20, 55, 11, 19, 15, 15, 11, 14, 11, 0, 7, 13, 53, 54, 17, 16, 
20, 16, 15, 16, 11, 11, 11, 16, 51, 48, 20, 18, 51, 13, 32, 32, 54, 20, 18, 51, 
54, 17, 0, 33, 33, 47, 53, 33, 48, 47, 20, 38, 11, 4, 41, 52, 55, 26, 52, 24, 53, 
54, 54, 6, 11, 50, 17, 53, 14, 54, 7, 37, 55, 48, 51, 54, 45, 48, 41, 50, 0, 41, 
47, 41, 42, 51, 46, 50, 47, 47, 47, 54, 53, 53, 0, 53, 54, 54, 43, 20, 11, 0, 0, 
49, 39, 18, 15, 20, 49, 15, 11, 16, 0, 18, 12, 18, 22, 21, 12, 50, 8, 44, 29, 20, 
54, 38, 12, 10, 20, 0, 46, 0, 51, 46, 40, 0, 18, 6, 0, 0, 9, 6, 0, 0, 20, 53, 19, 
0, 49, 45, 30, 0, 0, 17, 45, 15, 45, 49, 46, 0, 49, 21, 0, 10, 51, 47, 47, 10, 0, 
49, 0, 21, 0, 50, 0, 15, 19, 46, 0, 16, 27, 17, 22, 19, 49, 54, 12, 0, 27, 18, 49, 
45, 49, 49, 0, 54, 6, 0, 47, 0, 0, 0, 56, 0, 55, 57, 51, 0, 0, 50, 50, 34, 45, 8, 
54, 5, 53, 0, 0, 56, 56, 25, 35, 50, 56, 0, 55, 54, 25, 48, 2, 42, 11, 52, 55, 3, 
0, 7, 1, 8, 53, 56, 52, 56, 56, 56, 56, 55, 57, 6, 55, 7, 55, 8, 7, 54, 36, 57, 7, 
39, 56, 57, 53, 55, 49, 9, 46, 0, 30, 56, 57, 57, 56, 55, 55, 57, 30, 30, 5, 53, 
6, 0, 44, 30, 30, 0, 56, 12, 31, 56, 55, 56, 54, 54, 57, 54, 38, 38, 54, 54, 57, 
57, 44, 55, 55, 56, 0, 54};

/////////////////////////////////////////////////////////////////////////

	double right[N2]={
20, 12, 20, 25, 0, 26, 21, 6, 0, 0, 0, 0, 13, 0, 8, 0, 7, 26, 27, 2, 9, 29, 21, 
0, 29, 0, 15, 33, 0, 22, 0, 20, 19, 22, 0, 11, 0, 17, 0, 15, 0, 19, 10, 31, 10, 
0, 19, 0, 20, 0, 10, 6, 0, 0, 0, 20, 24, 22, 23, 20, 13, 10, 0, 0, 24, 22, 0, 15, 
20, 0, 30, 30, 21, 22, 0, 30, 0, 0, 0, 16, 33, 24, 19, 0, 31, 0, 22, 0, 0, 0, 0, 
0, 27, 19, 23, 0, 27, 0, 0, 0, 19, 0, 19, 14, 17, 21, 0, 23, 0, 0, 0, 3, 0, 0, 0, 
32, 24, 22, 16, 24, 0, 20, 0, 0, 0, 0, 7, 0, 0, 25, 20, 0, 0, 22, 33, 11, 8, 8, 
25, 0, 34, 0, 0, 0, 0, 0, 12, 23, 0, 20, 10, 19, 0, 0, 0, 8, 27, 0, 0, 0, 36, 16, 
23, 0, 0, 0, 0, 0, 0, 0, 42, 29, 43, 0, 0, 0, 0, 0, 0, 0, 0, 0, 49, 0, 0, 0, 0, 
52, 0, 0, 55, 0, 24, 0, 0, 18, 10, 15, 4, 20, 6, 16, 19, 16, 0, 24, 22, 29, 0, 
19, 18, 16, 16, 29, 0, 18, 0, 19, 0, 0, 0, 18, 20, 20, 29, 0, 0, 0, 0, 0, 0, 0, 
24, 15, 20, 0, 23, 20, 24, 26, 24, 0, 16, 24, 20, 20, 15, 17, 16, 30, 14, 17, 0, 
0, 22, 20, 24, 20, 20, 20, 15, 15, 19, 20, 0, 0, 24, 22, 0, 19, 0, 0, 0, 32, 24, 
0, 0, 22, 29, 0, 0, 0, 0, 0, 0, 0, 28, 0, 15, 33, 0, 0, 0, 31, 0, 28, 0, 0, 0, 
16, 20, 0, 26, 0, 17, 0, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 41, 0, 0, 0, 0, 0, 0, 0, 
0, 0, 0, 0, 0, 0, 48, 0, 0, 0, 0, 29, 21, 29, 26, 0, 0, 29, 26, 29, 0, 23, 15, 
25, 13, 29, 26, 23, 25, 26, 18, 0, 24, 0, 0, 25, 0, 0, 21, 23, 30, 10, 0, 14, 0, 
0, 0, 12, 21, 10, 10, 21, 27, 30, 28, 27, 30, 0, 23, 30, 0, 0, 0, 30, 11, 27, 0, 
29, 0, 0, 0, 12, 0, 26, 12, 20, 0, 0, 0, 26, 18, 0, 31, 27, 25, 0, 31, 20, 26, 0, 
11, 32, 0, 24, 32, 23, 0, 0, 24, 26, 33, 34, 0, 0, 0, 0, 37, 0, 31, 12, 0, 43, 
41, 30, 0, 30, 0, 0, 0, 30, 30, 0, 0, 0, 0, 30, 0, 30, 0, 31, 31, 0, 0, 31, 0, 
0, 0, 35, 0, 0, 40, 0, 40, 0, 41, 0, 0, 33, 47, 49, 6, 29, 0, 0, 0, 0, 0, 0, 0, 
0, 0, 29, 0, 29, 0, 29, 29, 0, 0, 0, 28, 0, 0, 0, 0, 0, 0, 29, 0, 30, 0, 0, 0, 
0, 0, 0, 0, 0, 0, 0, 30, 0, 30, 29, 0, 0, 0, 30, 0, 21, 0, 0, 0, 0, 0, 0, 0, 0, 
0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 44, 0};

/////////////////////////////////////////////////////////////////////////

	double z[N2]={
1, 3, 1, 1, 1, 1, 1, 3, 0, 0, 0, 0, 3, 0, 3, 2, 3, 1, 1, 3, 3, 1, 1, 0, 1, 1, 2, 
0, 0, 1, 0, 2, 2, 1, 0, 3, 0, 1, 1, 1, 0, 3, 3, 1, 3, 0, 3, 0, 2, 0, 2, 2, 0, 1, 
1, 1, 2, 1, 3, 3, 2, 3, 0, 0, 3, 1, 0, 3, 3, 1, 0, 1, 1, 2, 1, 2, 0, 0, 0, 2, 2, 
3, 3, 0, 1, 1, 3, 0, 2, 0, 0, 0, 1, 2, 2, 0, 3, 1, 0, 1, 1, 1, 3, 3, 1, 1, 1, 1, 
0, 0, 0, 2, 2, 0, 2, 1, 2, 2, 3, 1, 1, 3, 0, 0, 0, 0, 3, 0, 0, 3, 1, 0, 0, 1, 1, 
3, 3, 3, 2, 0, 1, 0, 0, 0, 0, 0, 2, 2, 0, 3, 3, 3, 1, 1, 1, 3, 3, 2, 0, 0, 1, 3, 
3, 0, 0, 0, 0, 0, 0, 0, 2, 1, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 0, 
0, 1, 0, 2, 0, 0, 2, 2, 2, 0, 2, 2, 1, 2, 3, 0, 0, 2, 1, 0, 1, 2, 3, 1, 1, 0, 2, 
0, 0, 0, 0, 0, 2, 1, 2, 2, 0, 0, 0, 0, 0, 1, 1, 2, 2, 2, 1, 2, 2, 0, 2, 2, 2, 2, 
2, 2, 2, 1, 2, 2, 2, 2, 2, 0, 0, 2, 2, 1, 2, 1, 2, 2, 2, 2, 2, 0, 0, 2, 0, 0, 2, 
1, 0, 1, 0, 0, 1, 0, 0, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0,
0, 0, 2, 2, 0, 1, 0, 1, 0, 2, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 
0, 0, 0, 0, 1, 0, 2, 0, 0, 0, 0, 2, 2, 3, 1, 1, 1, 3, 3, 3, 0, 3, 3, 3, 3, 1, 3, 
2, 1, 2, 3, 1, 2, 0, 0, 3, 1, 1, 2, 3, 1, 3, 1, 2, 1, 0, 0, 2, 2, 2, 3, 2, 0, 1, 
3, 3, 2, 1, 3, 1, 0, 1, 1, 2, 3, 3, 0, 2, 1, 0, 3, 3, 0, 1, 3, 2, 2, 0, 0, 2, 2, 
0, 3, 0, 0, 2, 3, 3, 2, 0, 3, 1, 0, 1, 3, 3, 0, 3, 3, 3, 0, 0, 1, 0, 0, 3, 1, 0, 
0, 3, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 2, 1, 2, 0, 1, 1, 0, 1, 1, 2, 0, 0, 0, 0, 0, 
0, 0, 0, 0, 0, 0, 0, 2, 0, 1, 1, 0, 1, 1, 1, 2, 2, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 
0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1, 2, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 
1, 0, 1, 2, 0, 0, 0, 0, 2, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 
0, 0, 1, 0};

	for (i=0;i<N2;i++)    
	{                  
		matrix[i][0] = z[i] + 1;	/* First Column (0): Covariate */
		matrix[i][1] = left[i];		/* Second Column (1): Left Endpoint */
		if (right[i] < 0.5)			/* Third Column (2): Right Endpoint */
		{
			matrix[i][2] = 60;		/* If 0 then right censored. So change the right endpoint to 100 as infinity*/
		}
		else
		{
			matrix[i][2] = right[i];
		}
	}
}

void BOOTS(register double newdata[N2][3], register double data[N2][3])
/*********************************************************
This is the bootstrap algorithm for generating new data
*********************************************************/
{
	int i, j, num;
	for (i=0;i<N2;i++)		/* The new data set has the same sample size */
	{
		num = static_cast<int>(rand()/60.23346); /* this generate an integer from 0 to 543 */
		for (j=0;j<3;j++)
		{
			newdata[i][j] = data[num][j];	  /* the ith row of newdata is one of the 544 rows of matrix */	
		}
	}
}

void SORTDATAINC(register double data[N2][3], int lower, int upper)
{
/************************************************************************************************************
This function, together with the function PARTITION2, sorts "data" by the observed survival time, 
X (with PARTITION2). Quicksort algorithm is used here. The purpose here is to put all censored observations 
in the end and all uncensored observations at beginning, and sorted. This algorithm is taken from a textbook 
("Algorithm Development and Program Design Using C" by Gary J. Bronson (1996), Pg 384).
************************************************************************************************************/
	int index;
	index = PARTITIONINC (data, lower, upper);
	if (lower < index)
		SORTDATAINC(data, lower, index-1);
	if (upper > index)
		SORTDATAINC(data, index+1, upper);
}

int PARTITIONINC(register double data[N2][3], int left, int right)
{
/************************************************************************************************************
This function, together with the function SORTDATA.INC, sorts "data" by the covariate, Z. Quicksort 
algorithm is used here. Notice that Z is in the first column (indexed by 0). This algorithm is taken 
from a textbook ("Algorithm Development and Program Design Using C" by Gary J. Bronson (1996), Pg 384).
data: 1st[0]:Z;  2nd[1]:Left;  3rd[2]:Right 	 

	SMALL ----> LARGE	Increasing
************************************************************************************************************/
	double pivot, temp1, temp2;
	pivot = data[left][0];
	temp1 = data[left][1];
	temp2 = data[left][2];
	while (left < right)
	{
		while (data[right][0] >= pivot && left < right)  /* scan from right, skip larger or equal values */
		{
			right--;
		}
		if (right != left)
		{
			data[left][0] = data[right][0];
			data[left][1] = data[right][1];
			data[left][2] = data[right][2];
			left ++;
		}
		while (data[left][0] <= pivot && left < right)  /* scan from left, skip smaller or equal values */
		{
			left ++;
		}
		if (right != left)
		{
			data[right][0] = data[left][0];
			data[right][1] = data[left][1];
			data[right][2] = data[left][2];
			right --;
		}
	}
	data[left][0] = pivot;
	data[left][1] = temp1;
	data[left][2] = temp2;
	return(left);
}

