#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#define N1 1000   /* Number of replicates */
#define N2 200   /* Sample size. Note: N2 must be an even number! */
#define N3 95    /* Number of potential distinct time points. Note: N3 depends on CENSORINGTIME */
#define CENSORINGTIME 88   /* Note: if this value changes, N3 must be changed correspondingly! */
#define TRUEBETA 1.0         /* True Beta */

/**********************************************************
True Beta = 1.0
[1,]   0.1000  0.2000  0.30000  0.40000  0.5000
[2,] 203.1593 88.2005 50.34938 31.78895 20.7247
**********************************************************/
/**********************************************************
True Beta = 0.5
[1,]   0.1000  0.20000  0.3000  0.40000  0.5000
[2,] 144.8267 64.32324 37.3649 23.92727 15.9431
**********************************************************/
/**********************************************************
True Beta = 0.0
[1,]   0.1000  0.20000  0.30000  0.40000  0.50000
[2,] 112.0741 49.89066 29.05494 18.65541 12.49824
**********************************************************/
/**********************************************************
True Beta = -0.5
[1,]  0.10000  0.20000  0.30000  0.40000 0.50000
[2,] 90.13384 39.84248 23.15489 14.81183 9.78568
**********************************************************/
/**********************************************************
True Beta = -1.0
[1,]  0.10000  0.20000  0.30000  0.40000 0.500000
[2,] 74.87172 32.78261 18.38723 11.51628 7.510695
**********************************************************/

/* Function Prototypes */

void HHAT(register double [N2][5],register double[N3][2],register double[N3][2]);
void INTEGRAL(register double [N2][N2],register double [N2][5],register double[N3][2],register double[N3][2],register double ahat[N2]);
void F1_F2(double [2],double,register double [N2][N2],register double [N2][5]);
double HHH(register int,register double,register double [N2][5],register double[N3][2],register double[N3][2]);
double GAUSSNEWTON(double,register double[N2][N2],register double [N2][5]);
double COMPUTES(double, double, double[N2], double[N2][5], double[N3][2], double[N3][2]);
void COMPUTEF(double[2], double[N2][5], double, double, double);
double MAKEDATA(register double [N2][5]);
double MIN(double,double); 
double MAX(double,double);
double ROUND(double,int);

void main(void)
{
	srand((unsigned)time(NULL));
	FILE *result;
	result=fopen("output.txt","w"); /* Result is printed to "output.txt" */

	register int i,j;
	double time;                    /* To be used in Summary 3 */
	double beta, est[N1], ests[N1+1][N3]; /* Results (beta) are stored in the matrix "est" */
				  					      /* Results (S_0hat) and the true value are stored in the matrix "ests" */
	register double data[N2][5];    /* Simulated data are stored in "data" */
	register double hhat0[N3][2];   /* Turnbull estimate (z=0) are stored in "hhat0" */
	register double hhat1[N3][2];   /* Turnbull estimate (z=1) are stored in "hhat1" */

	/* Note: hhat0 and hhat1 are BIG H's. I.e., distribution functions. hhat is small h */

	register double ahat[N2];       /* Right-Left of hhat0 or hhat1, depending on Z */
	register double hij[N2][N2];   /* Numerical integration result */

	fprintf(result,"   Beta    ||  Time Points (totally %d)\n", N3);
	fprintf(result," ==========|| ========================== \n  ");
	fprintf(result,"         ||  ");
	for (i=0;i<N3;i++)
	{
		ests[0][i]=i;
		fprintf(result,"%1.8f  ", ests[0][i]);	
	}
	fprintf(result,"           ||  \n");
	fprintf(result,"           ||  \n");
	double sum1=0.0,sum2=0.0,cenper=0.0;
	int count1=0, count2=0;
	for (i=1;i<=N1;i++)
	{
		count1++;						/* count1: how many replications are actually done */
		printf("%d\n",i);
		cenper=cenper+MAKEDATA(data);   /* Creat data, and store the censoring percentage */
										/* Now "data" is an N2 * 3 matrix with simulated observation */
		HHAT(data,hhat0,hhat1);			/* "hhat0' and "hhat1" store the Turnbull estimates */
		for (j=0;j<N2;j++)				/* "ahat" stores the Right-Left Turnbull estimates */
		{
			ahat[j]=HHH(j,data[j][2],data,hhat0,hhat1)-HHH(j,data[j][1],data,hhat0,hhat1);
		}
		INTEGRAL(hij,data,hhat0,hhat1,ahat);      /* Numerical integration, see paper for detail */ 
		beta=GAUSSNEWTON(TRUEBETA,hij,data);      /* Gauss-Newton algorithm is used to find the minimizer, TRUEBETA is the initial value */
		if (fabs(beta-TRUEBETA)>10.0) i--;
		else
		{
			count2++;
			if (beta <0)	fprintf(result,"%2.6f  ||  ",beta);
			else	fprintf(result,"%2.7f  ||  ",beta);
			est[i-1]=beta; 
			sum1=sum1+est[i-1];
			ests[i][0]=1;
			fprintf(result,"%1.8f  ", ests[i][0]);
			for (j=1;j<N3;j++)
			{
				ests[i][j]=COMPUTES(ests[0][j], beta, ahat, data, hhat0, hhat1);
				fprintf(result,"%1.8f  ",ests[i][j]);
			}
			fprintf(result,"\n");
		}
	}
	fprintf(result,"\n\n");
	fprintf(result,"         Summary 1      \n");
	fprintf(result,"=========================\n");
	fprintf(result,"Proportional odds Model\n");
	fprintf(result,"Sample size is %d;   %d replications\n",N2, N1);
	fprintf(result,"count1 (total) =%d, count2 (good) =%d\n",count1,count2);
	fprintf(result,"censoring ratio is %f\n\n\n",cenper/N1/1.0);

	fprintf(result,"         Summary 2      \n");
	fprintf(result,"=========================\n");
	fprintf(result,"True Beta is: %2.2f\n", TRUEBETA);
	fprintf(result,"Mean(Beta) = %f\nBias = %f\n\n\n",sum1/N1,sum1/N1-TRUEBETA);

	fprintf(result,"                               Summary 3      \n");
	fprintf(result,"====================================================================\n");
	fprintf(result,"  Time          S_0       EstS_0        Bias       Percent      Var\n");

	sum1=sum2=0;
	for (i=0;i<N3;i++)
	{
		sum1=sum2=0;
		for (j=1;j<=N1;j++)
		{
			sum1=sum1 + ests[j][i];
			sum2=sum2 + ests[j][i] * ests[j][i];
		}
		time=ests[0][i];
		fprintf(result, "%f	%f	%f	%f	%f	%f\n", 
	            time, 1/(1+0.08*time), sum1/N1, (sum1/N1-1/(1+0.08*time)), 
				fabs(sum1/N1-1/(1+0.08*time))*(1+0.08*time), (sum2-sum1*sum1/N1)/(N1-1));
	}
}

double COMPUTES(double t, double beta, double ahat[N2], 
				double data[N2][5], double hhat0[N3][2], double hhat1[N3][2] )
{
/************************************************************
This function calculates the estimator of survival function
************************************************************/
	int i, count = 0;
	double A=0, Ff[2], diff = 1, S, h = 0, hnew;
	for (i=0;i<N2;i++)
	{
		if ( t > data[i][2] )
		{
			A = A + 1.0;
		}
		else if ( t > data[i][1] )
		{
			if (ahat[i]>0)
			{
				A = A + (HHH(i,t,data,hhat0,hhat1)-HHH(i,data[i][1],data,hhat0,hhat1))/ahat[i];
			}
		}
	}
	if (A >= N2 )	S = 0;
	else if (A <= 0 )	S = 1;
	else
	{
		while (diff > 0.000000000001 && count < 20)
		{
			count = count + 1;
			COMPUTEF(Ff, data, beta, h, A);
			hnew = h - Ff[0]/Ff[1];
			diff = (hnew-h)*(hnew-h);
			h = hnew;
		}
		S = 1 / ( 1+ exp(h));
	}
	return(S);
}
	
void COMPUTEF(double Ff[2], double data[N2][5], double beta, double h, double A)
/***************************************************************
This function calculates the first derivative of F, which is f
***************************************************************/
{
	int i;
	double F= 0 - A, f=0;
	for (i=0;i<N2;i++)
	{
		F = F + 1 / ( 1 + exp( - h + data[i][0]*beta ) ) ;
		f = f + 1 / ( 2 + exp( - h + data[i][0]*beta ) + exp( h - data[i][0]*beta ) ) ;
	}
	Ff[0]=F;
	Ff[1]=f;
}	
	
double GAUSSNEWTON(double initial,register double hij[N2][N2],register double data[N2][5])
{
/**********************************************************
This function uses Gauss-Newton algorithm and returns the
minimizer of the sum of squares.
**********************************************************/
	double value[2];                   /* First and second derivatives are stored in "value" */
	double diff=1,beta0=initial,beta1;
	int count=0;
	while (diff>0.00000001 && count<10)
	{
		count++;
		F1_F2(value,beta0,hij,data);   /* First and second derivatives are evaluated at beta0 */
		beta1=beta0-value[0]/value[1]; /* Update the estimate */
		diff=pow((beta1-beta0),2);     /* Compute the difference */
		beta0=beta1;
	}
	return(beta0);
}  


void F1_F2(double value[2],double beta,register double hij[N2][N2],register double data[N2][5])
{
/**********************************************************
This function does not return anything but assigns values 
of the first and second derivatives evaluated at "beta" to 
the existed array "value". The original function is:
1/(1+exp((Zj-Zi)*beta)).
**********************************************************/
	int i,j;
	register double x,y,z,diff,tau0,tau1,tau2,sum1=0.0,sum2=0.0;
	for(i=0;i<N2;i++)
	{
		for(j=0;j<N2;j++)
		{
			diff=data[j][0]-data[i][0];
			x=exp(beta*diff)-1.0;
			y=diff*beta*(x+1.0);
			tau0=(fabs(x)<=0.0001)?(0.5):(1/x*(y/x-1));
			tau1=(fabs(x)<=0.0001)?(-1.0/6.0):(-(x+1)*((x+1)*(diff*beta-2.0)+diff*beta+2.0)/pow(x,3));
			tau2=(fabs(x)<=0.0001)?(0.0):(-2.0*(x+1)*(3.0/pow(x,3)+1.0/2.0/pow(x,2)-(diff)*(2.0*x+3.0)/pow(x,4))-tau1);
			z=hij[i][j]-tau0;
			sum1=sum1-2.0*z*diff*tau1;
			sum2=sum2-2.0*pow(diff,2)*(z*tau2-pow(tau1,2));
		}
	}
	value[0]=sum1; value[1]=sum2; /* sum1 is the first derivative and sum 2 is the second */
}

void INTEGRAL(register double hij[N2][N2],register double data[N2][5],register double hhat0[N3][2],register double hhat1[N3][2],register double ahat[N2])
{
/**********************************************************
This function does not return anything but assigns the 
numerically integrated value to the existed matrix "hij".
See paper for detail.
**********************************************************/
	register int i, j, l;
	register double sum;
	for (i=0;i<N2;i++)
	{
		for (j=0;j<N2;j++)
		{
			sum=0.0;
			l=1;
			if (data[i][0]<=0.5)         /* For Z=0 */
                                         /* Note: if using data[n][0]=0 (for Z=0), error occurs!! */
                                         /* Reason: data[n][0] is a double number, not an integer. */
			{
				while (l<=hhat0[0][0] && data[i][1]>=hhat0[l][0])	l++;
				while (l<=hhat0[0][0] && data[i][2]>=hhat0[l][0])
				{
					sum=sum+( HHH(j,MIN(data[j][2],hhat0[l][0]),data,hhat0,hhat1)
							 -HHH(j,MIN(data[j][1],hhat0[l][0]),data,hhat0,hhat1) )
						   *( HHH(i,hhat0[l][0],data,hhat0,hhat1)
						     -HHH(i,hhat0[l-1][0],data,hhat0,hhat1) );
					l++;                 /* See paper for detail */
				}
			}
			else                         /* For Z=1 */
			{
				while (l<=hhat1[0][0] && data[i][1]>=hhat1[l][0])	l++;
				while (l<=hhat1[0][0] && data[i][2]>=hhat1[l][0])
				{
					sum=sum+( HHH(j,MIN(data[j][2],hhat1[l][0]),data,hhat0,hhat1)
							 -HHH(j,MIN(data[j][1],hhat1[l][0]),data,hhat0,hhat1) )
						   *( HHH(i,hhat1[l][0],data,hhat0,hhat1)
						     -HHH(i,hhat1[l-1][0],data,hhat0,hhat1) );
					l++;
				}
			}
			if (i!=j && ahat[i]>0 && ahat[j]>0) 
			{
				hij[i][j]=sum/(ahat[i]*ahat[j]);
				if (hij[i][j]>1.0) hij[i][j]=1.0;
				if (hij[i][j]<0.0) hij[i][j]=0.0;
			}
			else	hij[i][j]=0.0;       /* Elements in "hij" are no larger than one */
		}
	}
}

double HHH(register int n,register double t,register double data[N2][5],register double hhat0[N3][2],register double hhat1[N3][2])
{
/**********************************************************
This function returns the survival function value 
corresponding to obsevation n (left or right).
**********************************************************/
	register int j=1;
	register double value=0.0;
	if (data[n][0]<=0.5)           /* Note: if using data[n][0]=0 (for Z=0), error occurs!! */
                                   /* Reason: data[n][0] is a double number, not an integer. */
	{
		while (j<=hhat0[0][0] && t>hhat0[j][0])	j++;
		if (j>hhat0[0][0]) value=1.00;
		else if (t<hhat0[j][0]) value=hhat0[j-1][1];
		else value=hhat0[j][1];
	}
	else                          /* For Z=1 */
	{
		while (j<=hhat1[0][0] && t>hhat1[j][0])	j++;
		if (j>hhat1[0][0]) value=1.00;
		else if (t<hhat1[j][0]) value=hhat1[j-1][1];
		else value=hhat1[j][1];
	}
	return(value);
}

void HHAT(register double data[N2][5],register double hhat0[N3][2],register double hhat1[N3][2])
{
/**********************************************************
This function does not return anything but computes the 
Turnbull estimates for different groups (Z=0 or Z=1) and 
stores them in the existed matrices "hhat0" and "hhat1".
**********************************************************/
	register double time0[N3],time1[N3];
	register int i,j,j1,j2,jj1,jj2,y=static_cast<int>(CENSORINGTIME),indi;
	for (i=0;i<N3;i++)                     /* Initialization */
	{
		time0[i]=time1[i]=0.0;
		hhat0[i][0]=hhat0[i][1]=0.0;
		hhat1[i][0]=hhat1[i][1]=0.0;
	}

/* Part 1: Summarize all the time points into hhat0 and hhat1 */

	for (i=0;i<N2/2;i++)          /* j1 and jj1 are used for Z=0, j2 and jj2 are used for Z=1 */
	{                             /* Arrays "time0" and "time1" are to be filled with observations */
		j1=static_cast<int>(data[i][1]);		jj1=static_cast<int>(data[i][2]);
		j2=static_cast<int>(data[i+N2/2][1]);	jj2=static_cast<int>(data[i+N2/2][2]);
		if (j1>y)		time0[y+1]=CENSORINGTIME*2.0;
		else		time0[j1]=data[i][1];
		if (jj1>y)		time0[y+1]=CENSORINGTIME*2.0;
		else		time0[jj1]=data[i][2];
		if (j2>y)		time1[y+1]=CENSORINGTIME*2.0;
		else		time1[j2]=data[i+N2/2][1];
		if (jj2>y)		time1[y+1]=CENSORINGTIME*2.0;
		else		time1[jj2]=data[i+N2/2][2];
	}
	j1=j2=2;                      /* Important! the first rows of "hhat0" and "hhat1" carry 
                                     numbers of rows those two matrices have, the second rows
                                     of "hhat0" and "hhat1" are always zeros. So we begin with
                                     row 3 */
	for (i=1;i<N3;i++)
	{
		if(time0[i]!=0)           /* Transfer time from time0(1) to hhat0(1) */
		{
			hhat0[j1][0]=time0[i];
			j1++;
		}
		if(time1[i]!=0)
		{
			hhat1[j2][0]=time1[i];
			j2++;
		}
	}
	hhat0[0][0]=hhat0[0][1]=--j1; /* First rows store the length of hhat0 (hhat1) */
	hhat1[0][0]=hhat1[0][1]=--j2;
	for (i=1;i<N3;i++)
	{
		hhat0[i][1]=1.0/j1;
		hhat1[i][1]=1.0/j2;       /* Initialize the second columns of hhat0 and hhat1 */
	}

/* Part 2: Compute the indicator matrices "aaa0" and "aaa1" */

	int **aaa0=(int **)malloc(sizeof(int *)*(N2/2)),**aaa1=(int **)malloc(sizeof(int *)*(N2/2));
	for (i=0;i<N2/2;i++)
	{
		aaa0[i]=(int *)malloc(sizeof(int)*j1);
		aaa1[i]=(int *)malloc(sizeof(int)*j2);  /* Create 2 N2xj1(j2) matrices */
	}
	for (i=0;i<N2/2;i++)
	{
		indi=0;
		for (jj1=1;jj1<=j1;jj1++)
		{
			if (data[i][1]<hhat0[jj1][0] && data[i][2]>=hhat0[jj1][0]) {aaa0[i][jj1-1]=1;indi++;}
			else aaa0[i][jj1-1]=0;
		} 
		if (indi==0)
		{
			for (jj1=1;jj1<=j1;jj1++)	{aaa0[i][jj1-1]=1;}
		}
		indi=0;
		for (jj2=1;jj2<=j2;jj2++)
		{
			if (data[i+N2/2][1]<hhat1[jj2][0] && data[i+N2/2][2]>=hhat1[jj2][0]) {aaa1[i][jj2-1]=1;indi++;}
			else aaa1[i][jj2-1]=0;
		}
		if (indi==0)
		{
			for (jj2=1;jj2<=j2;jj2++)	{aaa1[i][jj2-1]=1;}
		}
	}

/* Part 3: Turnbull algorithm, for computing survival functions */

	double *temp=(double *)malloc(sizeof(double)*(N2/2)), x, diff=1.0, sum;
	while (diff>0.0000000001)                       /* For Z=0 */
	{
		diff=0.0;
		for (i=0;i<N2/2;i++)
		{
			sum=0.0;
			for (j=0;j<j1;j++)
			{
				sum=sum+aaa0[i][j]*hhat0[j+1][1]; /* Matrix multiplication: (N2xj1)X(j1x1) */
			}
			temp[i]=sum;
		}

		for (i=0;i<j1;i++)
		{
			sum=0.0;
			for (j=0;j<N2/2;j++)
			{
				sum=sum+aaa0[j][i]/temp[j];
			}
			x=sum*hhat0[i+1][1]*2/N2;             /* x is the new (updated) estimator */
			diff=diff+pow((x-hhat0[i+1][1]), 2);
			hhat0[i+1][1]=x;
		}
	}
	for (i=2;i<=j1;i++)
	{
		hhat0[i][1]=hhat0[i][1]+hhat0[i-1][1];    /* Compute the cumulative (cdf) */
		if (hhat0[i][1]>1.0) hhat0[i][1]=1.0;
		if (hhat0[i][1]<0.0) hhat0[i][1]=0.0;
	}
	diff=1.0;
	while (diff>0.0000000001)                       /* For Z=1 */
	{
		diff=0.0;
		for (i=0;i<N2/2;i++)
		{
			sum=0.0;
			for (j=0;j<j2;j++)
			{
				sum=sum+aaa1[i][j]*hhat1[j+1][1];
			}
			temp[i]=sum;
		}

		for (i=0;i<j2;i++)
		{
			sum=0.0;
			for (j=0;j<N2/2;j++)
			{
				sum=sum+aaa1[j][i]/temp[j];
			}
			x=sum*hhat1[i+1][1]*2/N2;
			diff=diff+pow((x-hhat1[i+1][1]), 2);
			hhat1[i+1][1]=x;
		}
	}
	for (i=2;i<=j2;i++)
	{
		hhat1[i][1]=hhat1[i][1]+hhat1[i-1][1];
		if (hhat1[i][1]>1.0) hhat1[i][1]=1.0;
		if (hhat1[i][1]<0.0) hhat1[i][1]=0.0;
	}
}

double MAKEDATA(register double matrix[N2][5])
{
/**********************************************************
This function does not return anything but modifies the
existed matrix (array) called "data". The matrix "data" 
contains three columns and N2 rows (which means sample size
is N2). The first column is the covariate, Z and the next 
two columns contain the left and the right bounds of the 
observation time respectively. Notice that the censoring
ratio is controlled by CENSORINGTIME.
**********************************************************/
	int i;
	double ti1,ti2, count=0, temp=0;
	for (i=0;i<N2/2;i++)                 /* rows 0-(N2/2--1) have covariate 0 */
	{                                    /* rows (N2/2)--N2 have covariate 1  */
		ti1=(32769.0/(rand()+1)-1)*12.5;
		ti2=(32769.0/(rand()+1)-1)*12.5*exp(TRUEBETA);     /* PH model with beta=1 (e=2.7182..) */
		if (ti1>=CENSORINGTIME)
		{
			count=count+1;
			matrix[i][0]=0;
			matrix[i][1]=ROUND(CENSORINGTIME, 0);     /* precision: integers */
			matrix[i][2]=ROUND(CENSORINGTIME*2.0, 0);
		}
		else
		{
			matrix[i][0]=0;
			matrix[i][1]=ROUND(MAX(ti1-1-2*(rand()+1)/32769.0, 0),0);
			matrix[i][2]=ROUND(MIN(ti1+1+2*(rand()+1)/32769.0, CENSORINGTIME),0);
			temp = matrix[i][1];
			matrix[i][1] = ROUND(MAX (matrix[i][1], matrix[i][2] - 3), 0);
			matrix[i][2] = ROUND(MIN (matrix[i][2], temp + 3), 0);

		}                           
		if (ti2>=CENSORINGTIME)
		{
			count=count+1;
			matrix[i+N2/2][0]=1;
			matrix[i+N2/2][1]=ROUND(CENSORINGTIME, 0);
			matrix[i+N2/2][2]=ROUND(CENSORINGTIME*2.0, 0);
		}
		else
		{
			matrix[i+N2/2][0]=1;
			matrix[i+N2/2][1]=ROUND(MAX(ti2-1-2*(rand()+1)/32769.0, 0),0);
			matrix[i+N2/2][2]=ROUND(MIN(ti2+1+2*(rand()+1)/32769.0, CENSORINGTIME),0);
			temp = matrix[i+N2/2][1];
			matrix[i+N2/2][1] = ROUND(MAX (matrix[i+N2/2][1], matrix[i+N2/2][2] - 3), 0);
			matrix[i+N2/2][2] = ROUND(MIN (matrix[i+N2/2][2], temp + 3), 0);
		}
	}
	for (i=0;i<N2;i++)                 /* rows 0-(N2/2--1) have covariate 0 */
	{                                  /* rows (N2/2)--N2 have covariate 1  */
		if ( fabs(matrix[i][1]-matrix[i][2]) < 0.001)
		{
			matrix[i][1] = ROUND ( MAX (matrix[i][1]-2, 0) , 0) ;
			matrix[i][2] = ROUND ( MIN (matrix[i][2]+1, CENSORINGTIME), 0);
		}
	}
	return(count/N2/1.0);
}

double MIN(double num1,double num2)
{
/********************************************************
This function returns the smaller value of num1 and num2. 
********************************************************/
	if (num1<=num2)
		return(num1);
	else
		return(num2);
}

double MAX(double num1,double num2)
{
/********************************************************
This function returns the larger value of num1 and num2. 
********************************************************/
	if (num1>=num2)
		return(num1);
	else
		return(num2);
}

double ROUND(double num1,int num2)
{
/********************************************************
This function rounds off a "double" number to the desired 
decimal place. "num1" is the target number and"num2", 
which is an integer, specifies the decimal place. This 
function works as follows:

	If num1=1.234, num2=1, then num3=123 and num4=12, so
	num5=123-120=3. Now since 3<5, the function returns
	value 12/10=1.2.

	If num1=1.264, num2=1, then num3=126 and num4=12, so
	num5=126-120=6. Now since 6>=5, the function returns
	value (12+1)/10=1.3.
********************************************************/
	int num3=static_cast<int>(num1*pow(10,(num2+1)));
	int num4=static_cast<int>(num1*pow(10,num2));
	int num5=num3-num4*10;
	if (num5>=5)
		return((num4+1)/pow(10,num2));
	else
		return((num4)/pow(10,num2));
}



